/* ccgo: view.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "view.hh"
#include "root_proper.hh"
#include <gtkmm/spinbutton.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/box.h>
#include <gtkmm/alignment.h>
#include <gtkmm/stock.h>
#include <gtkmm/image.h>
#include <sstream>

using namespace go;

void View::change_position()
{
	unsigned p = unsigned(p_adj->get_value());
	while (walk.pos_num() < p) forward();
	while (walk.pos_num() > p) backward();
}

void View::set_range()
{
	int r = tail.pos_num();
	p_adj->set_upper(r);
	std::ostringstream s;
	s << r;
	n_move.set_text(s.str());
	p_adj->set_value(walk.pos_num());
	Walk w(walk);
	while (w.pos_num() && ! w.n()) if (! w.up()) break;
	p_adj->set_lower(w.pos_num());
}

void View::set_selection()
{
	if (! walk.n()) sel_a.set_sensitive(false);
	else sel_a.set_sensitive(true);
	if (! walk.n_hop() || walk.n() == walk.n_hop() - 1) sel_b.set_sensitive(false);
	else sel_b.set_sensitive(true);
}

void View::select_a()
{
	if (walk.n() == 0) return;
	int n = walk.n() - 1;
	walk.up();
	while (tail != walk) tail.up();
	walk.down(n);
	tail.down(n);
	while (tail.down());
	set_range();
	set_selection();
	walk.update_proper();
}

void View::select_b()
{
	if (! walk.n_hop() || walk.n() == walk.n_hop() - 1) return;
	int n = walk.n() + 1;
	walk.up();
	while (tail != walk) tail.up();
	walk.down(n);
	tail.down(n);
	while (tail.down());
	set_range();
	set_selection();
	walk.update_proper();
}

View::View(Node * n) :
	walk(n),
	tail(n),
	p_adj(Gtk::manage(new Gtk::Adjustment(0, 0, 0))),
	n_move("0")
{
	walk.start();
	title.set_text(walk.Game::get_name());
	// GUI setup
	Gtk::VBox * vb = Gtk::manage(new Gtk::VBox);
	pack1(* vb);
	vb->pack_start(title, Gtk::PACK_SHRINK);
	Gtk::HBox * hb = Gtk::manage(new Gtk::HBox);
	vb->pack_start(* hb);
	Gtk::HBox *hb2 = Gtk::manage(new Gtk::HBox);
	vb->pack_end(* hb2, Gtk::PACK_SHRINK);
	hb2->pack_start(walk.modifi_widget(), Gtk::PACK_SHRINK);
	hb2->pack_end(walk.label_widget(), Gtk::PACK_SHRINK);
	Gtk::ScrolledWindow * sw = new Gtk::ScrolledWindow;
	sw->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
	sw->add(walk.text_widget());
	pack2(* sw);
	hb->pack_start(walk);
	vb = Gtk::manage(new Gtk::VBox);
	hb->pack_end(* vb, Gtk::PACK_SHRINK);
	Gtk::SpinButton * sb = Gtk::manage(new Gtk::SpinButton(* p_adj, 0, 0));
	vb->pack_start(* Gtk::manage(new Gtk::Label("Node")), Gtk::PACK_SHRINK);
	vb->pack_start(n_move, Gtk::PACK_SHRINK);
	vb->pack_start(* sb, Gtk::PACK_SHRINK);
	Gtk::Alignment * al;
	al = Gtk::manage(new Gtk::Alignment(0.5, 0.5, 0, 0));
	vb->pack_start(* al, Gtk::PACK_SHRINK);
	hb = Gtk::manage(new Gtk::HBox);

	al->add(* hb);
	hb->pack_start(sel_a, Gtk::PACK_SHRINK);
	hb->pack_start(sel_b, Gtk::PACK_SHRINK);
	sel_a.add(* Gtk::manage(new Gtk::Image(Gtk::Stock::GO_BACK, Gtk::ICON_SIZE_SMALL_TOOLBAR)));
	sel_b.add(* Gtk::manage(new Gtk::Image(Gtk::Stock::GO_FORWARD, Gtk::ICON_SIZE_SMALL_TOOLBAR)));

	al = Gtk::manage(new Gtk::Alignment(0.5, 0.5, 0, 0));
	vb->pack_start(* al, Gtk::PACK_SHRINK);
	hb = Gtk::manage(new Gtk::HBox);
	al->add(* hb);
	hb->pack_start(* Gtk::manage(new Gtk::Label("W:")), Gtk::PACK_SHRINK);
	hb->pack_start(walk.w_score_widget(), Gtk::PACK_SHRINK);
	al = Gtk::manage(new Gtk::Alignment(0.5, 0.5, 0, 0));
	vb->pack_start(* al, Gtk::PACK_SHRINK);
	hb = Gtk::manage(new Gtk::HBox);
	al->add(* hb);
	hb->pack_start(* Gtk::manage(new Gtk::Label("B:")), Gtk::PACK_SHRINK);
	hb->pack_start(walk.b_score_widget(), Gtk::PACK_SHRINK);
	p_adj->signal_value_changed().connect(slot(* this, & View::change_position));
	sel_a.signal_clicked().connect(slot(* this, & View::select_a));
	sel_b.signal_clicked().connect(slot(* this, & View::select_b));
	vb->pack_end(button_area, Gtk::PACK_SHRINK);

	// get ready
	while (tail.down());
	set_range();
	set_selection();
	walk.update_proper(); // update game proper at the walk node
	// walk.set_tracking(false);
}

View::~View()
{
}

void View::forward()
{
	walk.down();
	set_selection();
	walk.update_proper();
}

void View::backward()
{
	int n = walk.n();
	walk.up();
	if (n != 0) {
		while (tail != walk) tail.up();
		while (tail.down());
		set_range();
	}
	set_selection();
	walk.update_proper();
}

Gtk::VBox & View::get_button_area()
{
	return button_area;
}
